/*
 * arigo.c
 *
 * LINUX device driver for the Echelon arigo Lonworks interface card.
 *
 * Copyright Signum Support AB 1998-
 *    Written by Christer Weinigel
 *
 *  Modified and updated by Gran Hasse
 *  gh@raditex.se 2000
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU Library General Public License as
 * published by the Free Software Foundation; either version 2 of the
 * License, or (at your option) any later version.  This library is
 * distributed in the hope that it will be useful, but WITHOUT ANY
 * WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE.  See the GNU Library General Public
 * License for more details.  You should have received a copy of the GNU
 * Library General Public License along with this library; if not, write
 * to the Free Software Foundation, 59 Temple Place - Suite 330, Boston,
 * MA 02111-1307, USA.
 *
 * Usage:
 *   insmod arigo.o base=0x310 irq=11 [debug=n]
 *
 * Miscellaneous notes:
 *
 * This adaptes uses the MIP/P20 or MIP/P50 Link-Layer Buffer format
 * as documented in Appendix C, Bit Field Diagrams from the LonWorks
 * Host Application Programmers Guide.
 *
 * As far as I can tell, the Network Interface Commands described in
 * Appendix D of the LonWorks Host Application Programmers Guide are
 * the ones pertaining to this adapter.  
 *
 * For a description and state diagram for the flow control commands,
 * see Chapter 8, Session Layer Protocol from the Serial LonTalk
 * Adapter and Serial Gateway User's Guide.
 * 
 */

#include <linux/module.h>
#include <linux/version.h>

#include <linux/errno.h>
#include <linux/kernel.h>
#include <linux/mm.h>
#include <linux/major.h>
#include <linux/sched.h>
#include <linux/ioport.h>
#include <linux/delay.h>

#include <asm/uaccess.h>

#include <asm/io.h>
#include <asm/irq.h>

/*
  TODO:

  sanitizie for more than one adapter
  fix the buffer handling
  send pause commands downlink when the uplink buffer is full
  clean up workproc
 */

#define NAME "arigo"
#define DEBUG			/* for debugging */

#define DIRECT 1

/* Miscellaneous */
typedef unsigned char byte;

#define MAX_DEV 4

#define CMD_niCOMM 0x10
#define CMD_niNETMGMT 0x20
#define CMD_RESET 0x50
#define CMD_FLUSH_CANCEL 0x60
#define CMD_SLEEP 0xb0
#define CMD_WRITE_MEMORY 0xd1
#define CMD_READ_MEMORY 0xd3
#define CMD_QUERY_BUFFERS 0xe7

unsigned int free_buf_translate[16] = { 0, 0, 1, 2, 3, 5, 7, 11, 15, 23, 31, 47, 63, 95, 127, 191 };

enum STATE { INIT, SLAVE, MASTER, FAILURE };

/* module parameters */
#ifdef DEBUG
static int debug = 0;
#ifdef KERNEL_2_2
MODULE_PARM(debug, "i");
#endif
#endif

static int major = 41;		/* Major device number */
#ifdef KERNEL_2_2
MODULE_PARM(major, "i");
#endif

#define BASE_NONE -1
#define IRQ_NONE -1

static int base[MAX_DEV] = { 0x210, BASE_NONE, BASE_NONE, BASE_NONE };
static int irq[MAX_DEV] = { 9, IRQ_NONE, IRQ_NONE, IRQ_NONE };
#ifdef KERNEL_2_2
MODULE_PARM(base, "1-" __MODULE_STRING(MAX_DEV) "i");
MODULE_PARM(irq, "1-" __MODULE_STRING(MAX_DEV) "i");
#endif

/* Device information */
static struct devinfo 
{
    int base;			/* I/O address base */
    int irq;			/* IRQ line */

    int open_count;		/*  */

    int initialized;		/*  */
    
    int acked;			/*  */

    enum STATE state;		/* Current state  */

    byte immediate_command;	/* For immediate commands */
    byte small_msg[8];		/* For small messages, usually an immediate */
    byte *msg_ptr;		/* Current message pointer */

    int query_buffers;		/* Query adapter about free buffers */

    int downlink_size;		/* No of downlink buffers */
    int downlink_priority_size;	/* No of downlink priority buffers */

    int downlink_free;		/* Free downlink buffers */
    int downlink_priority_free;	/* Free downlink priority buffers */

    struct wait_queue *downlink_wq; /* Downlink semaphore */
    struct wait_queue *uplink_wq; /* Uplink semaphore */
} devinfo[MAX_DEV];

struct devinfo *irq_di[NR_IRQS]; /* IRQ to devinfo mapping */

static int major_allocated;

/* Interface name read from SIGSTART to SIGEND */
const char *signature = "ARIGO94";

/* Register addresses */
#define DATA(di)	((di).base)	/* Data register */
#define STATUS(di)	((di).base+2)	/* Status register */
#define COMMAND(di)	((di).base+2)	/* Command register */
#define IRQSTATUS(di)	((di).base+3)	/* Interrupt status registern */
#define IRQMASK(di)	((di).base+4)	/* Interrupt mask register */
#define DMASELECT(di)	((di).base+5)	/* DMA select register */
#define IRQSELECT(di)	((di).base+6)	/* Interrupt select register */
#define VERSION(di)	((di).base+7)	/* Hardware version register */
#define SIGSTART(di)	((di).base+8)	/* Interface signature starts */
#define LOGICALDEVNO(di) ((di).base+0xf) /* Device number register */

/* Status register definitions */
#define S_HS		0x01	/* Status of /HS */
#define S_RESET		0x02	/* Status of /RESET */
#define S_SERVICE	0x04	/* Status of /SERVICE */

/* Command register definiton */
#define C_START		0x01	/* Start interface (/RESET set high) */
#define C_STOP		0x02	/* Stop interface (/RESET set low) */
#define C_DMA_START	0x03	/* Start DMA (for DMA mode) */
#define C_DMA_MASTER	0x04	/* Master read (for DMA mode) */
#define C_DMA_CLEAR	0x05	/* Clear end (for DMA mode) */
#define C_SET_SERVICE	0x06	/* Set service pin (/SERVICE set low) */
#define C_CLEAR_SERVICE	0x07	/* Clear service pin (/SERVICE set high) */
#define C_SET_DOWN	0x08	/* Set down request */
#define C_CLEAR_DOWN	0x09	/* Clear down request */
#define C_RESET		0x0a	/* Reset ALTERA and NEURON chip */
#define C_IRQTEST	0x0b	/* Interrupt test, trigger an interrupt */

/* Interrupt status register and Interrupt mask register */
#define I_ERROR		0x01	/* Hardware error */
#define I_DMA		0x02	/* DMA transfer end */
#define I_UPLOAD	0x04	/* Upload request */
#define I_DOWNLOAD	0x08	/* Download request */

#define CMD_RESYNC	0x5a	/*  */
#define CMD_ACKSYNC	0x07	/*  */
#define CMD_XFER	0x01
#define CMD_NULL	0x00

/* Queue Types */
#define niTYPE			0xF0	/* type mask */

#define niLOCAL_CMD		0x00	/* local command */
#define niTQ			0x02	/* trans queue */
#define niTQ_P			0x03	/* trans queue priority */
#define niNTQ			0x04	/* non-trans queue */
#define niNTQ_P			0x05	/* non-trans queue priority */
#define niRESPONSE		0x06	/* response queue */
#define niINCOMING		0x08	/* incoming queue */

#define niCOMM			0x10	/* outgoing message */
#define niNETMGMT		0x20	/* local net mgmt */

/* Command/queue byte values: */
#define niRESET			0x50
#define niFLUSH_CANCEL		0x60
#define niACK			0xC0
#define niNACK			0xC1
#define niPUPXOFF		0xE1
#define niPUPXON		0xE2    /* was 0xE4 */

/* Ring buffer stuff */
#define MSGLEN 256
#define MSGMAX 64

static byte downlink[MSGMAX][MSGLEN];
static int downlink_head, downlink_tail;
static byte uplink[MSGMAX][MSGLEN];
static int uplink_head, uplink_tail;

/* For debugging */
#ifdef DEBUG
void hd(char *info, byte *ptr, int count)
{
    char s[80], *p;
    int line, col;

    for (line = 0; line < count; line += 8)
    {
	p = s;

	p += sprintf(p, "%s", info);

	p += sprintf(p, "%04x  ", line);

	for (col = 0; col < 8; col++)
	{
	    if (line + col < count)
		p += sprintf(p, "%02x ", ptr[line + col]);
	    else
		p += sprintf(p, "   ");
	}

	*p++ = ' ';
	
	for (col = 0; col < 8; col++)
	{
	    if (line + col >= count)
		break;

	    if (ptr[line + col] >= ' ' && ptr[line + col] < 127)
		*p++ = ptr[line + col];
	    else
		*p++ = '.';
	}

	*p = '\0';

	printk("%s\n", s);
    }
}

#endif

static int wait_hs_low(struct devinfo *di)
{
    do
    {
	if (inb(IRQSTATUS(*di)) & I_ERROR)
	{ 
	    /* printk(KERN_ERR "wait_hs_low: I_ERROR set\n"); */
	    return -1;
	}
    }
    while (inb(STATUS(*di)) & S_HS);

    return 0;
}

static int do_downlink(struct devinfo *di, byte *buffer, int len)
{
    int n;

    for (n = 0; n < len; n++)
    {
	if (wait_hs_low(di) == -1)
	    return -1;
	outb(buffer[n], DATA(*di));
    }

    return n;
}

static int do_uplink(struct devinfo *di, byte *buffer, int len)
{
    int n;

    for (n = 0; n < len; n++)
    {
	if (wait_hs_low(di) == -1)
	    return -1;

	buffer[n] = inb(DATA(*di));
    }

    return n;
}

static void interrupt(int irq, void *dev_id, struct pt_regs *regs)
{
    struct devinfo *di = irq_di[irq]; /* find devinfo for this IRQ */
    byte scratch[256];
    int scratchlen;

    printk("Irq is %d\n", irq);

    if (di == NULL)
    {
	printk(KERN_WARNING NAME ": spurious interrupt %d\n", irq);
	return;
    }

    if (debug > 2)
	printk(KERN_DEBUG NAME ": STATE = %d, IRQSTATUS = 0x%x\n", di->state,
	       inb(IRQSTATUS(*di)));

    switch (di->state)
    {
    case INIT:
    {
	int sync_tries;
    
    do_init:
	if (debug > 1)
	    printk(KERN_DEBUG NAME ": init (IRQSTATUS = 0x%x)\n", 
		   inb(IRQSTATUS(*di)));

	sync_tries = 5;
	while (--sync_tries > 0)
	{
	    outb(C_STOP, COMMAND(*di));
	    
	    /* clear buffers  */
	    
	    outb(C_START, COMMAND(*di));
	    
	    outb(I_ERROR, IRQMASK(*di));
	
	    scratchlen = 0;
	    scratch[scratchlen++] = CMD_RESYNC;
	    scratch[scratchlen++] = 0;

	    if (debug > 2)
		hd(KERN_DEBUG NAME " INIT downlink: ", scratch, scratchlen);
	    if (do_downlink(di, scratch, scratchlen) == -1)
		continue;

	    if (do_uplink(di, scratch, 1) == -1)
		continue;
	    if (debug > 2)
		hd(KERN_DEBUG NAME " INIT uplink: ", scratch, 1);
	    
	    if (scratch[0] == CMD_ACKSYNC)
		break;
	} /* while (sync_tries > 0) */

	if (sync_tries == 0)
	    goto do_failure;

	if (!di->initialized)
	{
	    di->initialized = 1;

	    scratchlen = 0;
	    scratch[scratchlen++] = CMD_XFER;
	    scratch[scratchlen++] = 1;
	    scratch[scratchlen++] = 0x60;
	    scratch[scratchlen++] = 0;

	    if (debug > 2)
		hd(KERN_DEBUG NAME " INIT downlink2: ", scratch, scratchlen);
	    if (do_downlink(di, scratch, scratchlen) == -1)
		goto do_init;

	    if (wait_hs_low(di) == -1)
		goto do_init;
	
#if DIRECT
	    if (inb(IRQSTATUS(*di)) & (I_DOWNLOAD | I_UPLOAD))
		goto token_slave;
#endif

	    outb(I_ERROR | I_DOWNLOAD | I_UPLOAD, IRQMASK(*di));

	    di->state = SLAVE;
	}
	else /* !initialized */
	{
#if DIRECT
	    if (inb(IRQSTATUS(*di)) & (I_DOWNLOAD | I_UPLOAD))
		goto token_master;
#endif

	    outb(I_ERROR | I_DOWNLOAD | I_UPLOAD, IRQMASK(*di));

	    di->state = MASTER;
	}

	di->acked = 0;
	
	return;
    } /* case INIT: */

    case SLAVE:
    {
	int len;

	outb(I_ERROR, IRQMASK(*di));

    token_slave:
	if (debug > 1)
	    printk(KERN_DEBUG NAME ": slave (IRQSTATUS = 0x%x)\n", 
		   inb(IRQSTATUS(*di)));

	scratchlen = 0;
	scratch[scratchlen++] = inb(DATA(*di));

	switch (scratch[0])
	{
	case CMD_XFER:
	    if (wait_hs_low(di) == -1)
		goto do_init;

	    len = inb(DATA(*di));

	    scratch[scratchlen++] = len;
	    if (do_uplink(di, scratch + scratchlen, len) == -1)
		goto do_init;
	    scratchlen += len;

	    /* Do something with the packet */
	    if (len >= 1 && scratch[2] == niACK)
	    {
		outb(C_SET_DOWN, COMMAND(*di));
		di->acked = 1;
	    }
	    else
	    {
		/* TODO: buffer handling */
		int new_head;

		new_head = (uplink_head + 1) % MSGMAX;
		if (new_head == uplink_tail)
		    printk(KERN_ERR NAME ": uplink buffer full, dropping message\n");
		else
		{
		    memcpy(uplink[uplink_head], scratch + 1, len + 1);
		    uplink_head = new_head;

		    wake_up_interruptible(&di->uplink_wq);
		}

#if 0
		if (uplnk_full())
		    immediate_command = 0xe1; /* TODO: PAUSE? */
#endif
	    }
	    
	case CMD_NULL:
	    if (debug > 2)
		hd(KERN_DEBUG NAME " SLAVE uplink: ", scratch, scratchlen);

	    if (wait_hs_low(di) == -1)
		goto do_init;
	
#if DIRECT
	    if (inb(IRQSTATUS(*di)) & (I_DOWNLOAD | I_UPLOAD))
		goto token_master;
#endif
	    
	sleep_master:
	    outb(I_ERROR | I_DOWNLOAD | I_UPLOAD, IRQMASK(*di));
	    
	    di->state = MASTER;
	    
	    return;
	    
	default:
	    printk(KERN_ERR NAME ": bad command 0x%x\n", scratch[0]);
	    goto do_init;
	}
    }

    case MASTER:
	outb(I_ERROR, IRQMASK(*di));

    token_master:
	if (debug > 1)
	    printk(KERN_DEBUG NAME ": master (IRQSTATUS = 0x%x)\n", 
		   inb(IRQSTATUS(*di)));

	/* Are there any downlink messages in the queue? */
	if (downlink_head != downlink_tail)
	{
	    int len;
	    int cmd;

	    scratchlen = 0;
	    scratch[scratchlen++] = CMD_XFER;

	    len = downlink[downlink_tail][0]+1;

	    if (len >= 1)
		cmd = downlink[downlink_tail][1];
	    else
		cmd = -1;

	    if (di->acked
		|| ((cmd & niTYPE) != niCOMM && (cmd & niTYPE) != niNETMGMT))
	    {
		memcpy(scratch + scratchlen, downlink[downlink_tail], len);
		scratchlen += len;
	    
		downlink_tail = (downlink_tail + 1) % MSGMAX;

		wake_up_interruptible(&di->downlink_wq);

		di->acked = 0;
	    }
	    else
	    {
		scratch[scratchlen++] = 1; /* length */
		scratch[scratchlen++] = cmd;

#if 1
		outb(C_CLEAR_DOWN, COMMAND(*di));
#endif
	    }

	    scratch[scratchlen++] = 0;
	}
	else
	{
	    outb(C_CLEAR_DOWN, COMMAND(*di));

#if DIRECT
	    if (!(inb(IRQSTATUS(*di)) & I_UPLOAD))
		goto sleep_master;
#endif

	    scratchlen = 0;
	    scratch[scratchlen++] = CMD_NULL;
	    scratch[scratchlen++] = 0;
	}

	if (debug > 2)
	    hd(KERN_DEBUG NAME " MASTER downlink: ", scratch, scratchlen);
	if (do_downlink(di, scratch, scratchlen) == -1)
	    goto do_init;

	if (wait_hs_low(di) == -1)
	    goto do_init;
	
#if DIRECT
	if (inb(IRQSTATUS(*di)) & (I_DOWNLOAD | I_UPLOAD))
	    goto token_slave;
#endif
	
	outb(I_ERROR | I_DOWNLOAD | I_UPLOAD, IRQMASK(*di));
	
	di->state = SLAVE;
	
	return;

    case FAILURE:
    do_failure:
        printk(KERN_ERR NAME ": failure\n");

	outb(0, IRQMASK(*di));
	outb(C_STOP, COMMAND(*di));

	di->state = FAILURE;

	return;
    }
}

/* Try to detect a an adapter */
int detect(struct devinfo *di)
{
    int i;
    int siglen;

    siglen = strlen(signature);

    outb(C_STOP, COMMAND(*di));
    outb(C_RESET, COMMAND(*di));
    inb(VERSION(*di));

    for (i = 0; i < siglen; i++)
    {
	if (inb(SIGSTART(*di) + i) != signature[i])
	{
	    printk(KERN_WARNING NAME ": invalid signature\n");
	    return -ENODEV;
	}
    }

    if (inb(DATA(*di)) != 0
	|| inb(DATA(*di) + 1) != 0
	|| inb(STATUS(*di)) != (S_HS | S_RESET)
	|| inb(IRQSTATUS(*di)) != 0
	|| inb(IRQMASK(*di)) != 0
	|| inb(IRQSELECT(*di)) != 0)
    {
	printk(KERN_WARNING NAME ": bad register status\n");
	return -ENODEV;
    }

    outb(C_STOP, COMMAND(*di));

    return 0;
}

int arigo_open(struct inode *inode, struct file *file)
{
    int minor = MINOR(inode->i_rdev);
    struct devinfo *di;

    if (minor > MAX_DEV)
	return -ENXIO;

    di = &devinfo[minor];
   
    if (di->base == BASE_NONE)
	return -ENXIO;

    MOD_INC_USE_COUNT;
  
    printk("The device is open\n");

    return 0;

}

int arigo_release(struct inode * inode, struct file * file)
{
    int minor = MINOR(inode->i_rdev);
    struct devinfo *di;

    di = &devinfo[minor];
   
    MOD_DEC_USE_COUNT;

    printk("The device have been closed\n");

    return 0;

}



static ssize_t arigo_read(struct file *file, 
			char * buf, size_t count, loff_t *ppos)
{
    struct devinfo *di;
    int r;
    int len;

#ifdef KERNEL_2_2
    di = &devinfo[MINOR(file->f_dentry->d_inode->i_rdev)];
#else
    di = &devinfo[MINOR(inode->i_rdev)];
#endif

    if ((r = verify_area(VERIFY_WRITE, buf, count)) != 0)
	return r;

    if (count == 0)
	return 0;

    if (uplink_head == uplink_tail)
    {
	long flags;

	if (file->f_flags & O_NONBLOCK)
	    return -EAGAIN;

	save_flags(flags);
	cli();
	do
	{
	    interruptible_sleep_on(&di->uplink_wq);
	    
	    /* sleep was interrupted by a signal */
#ifdef KERNEL_2_2
	    if (signal_pending(current))
#else
	    if (current->signal & ~current->blocked)
#endif
	    {
		restore_flags(flags);
		return -EINTR;
	    }
	}
	while (uplink_head == uplink_tail);
	restore_flags(flags);
    }
    
    len = uplink[uplink_tail][0]; /* length */
    uplink[uplink_tail][0] = uplink[uplink_tail][1]; /* command */
    uplink[uplink_tail][1] = len - 1;

    r = len + 1;

    if (r > count)
	r = count;

#ifdef KERNEL_2_2
    if (copy_to_user(buf, uplink[uplink_tail], r))
	return -EFAULT;
#else
    memcpy_tofs(buf, uplink[uplink_tail], r);
#endif

    uplink_tail = (uplink_tail + 1) % MSGMAX;

    return r;
}


static ssize_t arigo_write(struct file *file, 
			 const char *buf, size_t count, loff_t *ppos)
{
    struct devinfo *di;
    int r;
    int new_head;
    int len;
    byte blen;

#ifdef KERNEL_2_2
    di = &devinfo[MINOR(file->f_dentry->d_inode->i_rdev)];
#else
    di = &devinfo[MINOR(inode->i_rdev)];
#endif

    printk("Writing to device\n");

    if ((r = verify_area(VERIFY_READ, buf, count)) != 0)
	return r;

    if (count < 2 || count > MSGMAX)
	return -EINVAL;

#ifdef KERNEL_2_2
    if (copy_from_user(&blen, buf+1, 1))
	return -EFAULT;
#else
    blen = get_fs_byte(buf+1);
#endif    
    if (blen != count - 2)
	return -EINVAL;

    if (count == 0)
	return 0;

    new_head = (downlink_head + 1) % MSGMAX;
    if (new_head == downlink_tail)
    {
	long flags;

	if (file->f_flags & O_NONBLOCK)
	    return -EAGAIN;

	save_flags(flags);
	cli();
	do
	{
	    interruptible_sleep_on(&di->uplink_wq);

	    /* sleep was interrupted by a signal */
#ifdef KERNEL_2_2
	    if (signal_pending(current))
#else
	    if (current->signal & ~current->blocked)
#endif
	    {
		restore_flags(flags);
		return -EINTR;
	    }
	}
	while (new_head == downlink_tail);
	restore_flags(flags);
    }

#ifdef KERNEL_2_2
    if (copy_from_user(downlink[downlink_head], buf, count))
	return -EFAULT;
#else
    memcpy_fromfs(downlink[downlink_head], buf, count);
#endif

#ifdef DEBUG
    if (debug > 2)
	hd("arigo_write: ", downlink[downlink_head], count);
#endif

    len = downlink[downlink_head][1]; /* length */
    downlink[downlink_head][1] = downlink[downlink_head][0]; /* command */
    downlink[downlink_head][0] = len + 1;

    downlink_head = new_head;

    outb(C_SET_DOWN, COMMAND(*di));

    return count;
}

static struct file_operations arigo_fops = 
{
    NULL,			/* lseek, not needed for 2.1.x */
    arigo_read,
    arigo_write,
    NULL,			/* readdir */
    NULL,			/* select or poll */
    NULL,
    NULL,			/* mmap */
    arigo_open,
    NULL,
    arigo_release,
};

void cleanup_module(void)
{
    int i;

#ifdef DEBUG
    if (debug > 1)
	printk(KERN_DEBUG NAME ": cleanup_module()\n");
#endif

    for (i = 0; i < MAX_DEV; i++)
    {
	struct devinfo *di = &devinfo[i];

	if (di->base != BASE_NONE)
	{
	    di->state = FAILURE;
	
	    outb(C_STOP, COMMAND(*di));

	    outb(0, IRQMASK(*di));
	    outb(0, IRQSELECT(*di));
	}

	if (di->irq != IRQ_NONE)
	{
	    free_irq(di->irq, NULL);
	    di->irq = IRQ_NONE;

	    irq_di[di->irq] = NULL; /* clear irq -> devinfo mapping */
	}

	if (di->base != BASE_NONE)
	{
	    release_region(di->base, 16);
	    di->base = BASE_NONE;
	}
    }

    if (major_allocated)
    {
	unregister_chrdev(major, NAME);
	major_allocated = 0;
    }
}

static void init_devinfo(struct devinfo *di)
{
    di->base = BASE_NONE;
    di->irq = IRQ_NONE;

    di->open_count = 0;

    di->initialized = 0;

    di->state = FAILURE;

    di->immediate_command = 0;

    di->query_buffers = 0;

    di->downlink_size = 1;
    di->downlink_free = di->downlink_size;
    di->downlink_priority_size = 1;
    di->downlink_priority_free = di->downlink_priority_size;

    init_waitqueue(&di->downlink_wq);
    init_waitqueue(&di->uplink_wq);
}

int init_module(void)
{
    int r;
    int i;
    long flags;

    major_allocated = 0;

    for (i = 0; i < MAX_DEV; i++)
	init_devinfo(&devinfo[i]);
    for (i = 0; i < NR_IRQS; i++)
	irq_di[i] = NULL;
    
    downlink_head = downlink_tail = 0;
    uplink_head = uplink_tail = 0;

#ifdef DEBUG
    if (debug > 1)
	printk(KERN_DEBUG NAME ": init_module()\n");
#endif

    if (register_chrdev(41, NAME, &arigo_fops)) 
    {
	printk("unable to get major %d for %s\n", major, NAME);
	return -EAGAIN;
    }
    major_allocated = 1;

    for (i = 0; i < MAX_DEV; i++)
    {
	struct devinfo *di = &devinfo[i];
	
	if (base[i] == BASE_NONE || irq[i] == IRQ_NONE)
	    continue;

	if (check_region(base[i], 16) < 0)
	{
	    printk(KERN_ERR NAME ": unable to allocate I/O att address 0x%x - 0x%x\n",
		   base[i], base[i]+15);
	    cleanup_module();
	    return -EAGAIN;
	}
	request_region(base[i], 16, NAME);
	di->base = base[i];

	if (request_irq(irq[i], interrupt, SA_INTERRUPT, NAME, NULL))
	{
	    printk(KERN_ERR NAME "unable to allocate IRQ %d\n", irq[i]);
	    cleanup_module();
	    return -EAGAIN;
	}
	di->irq = irq[i];

	if ((r = detect(di)) != 0)
	{
	    cleanup_module();
	    return r;
	}

	irq_di[irq[i]] = di;

	save_flags(flags);
	cli();
    
	outb(C_STOP, COMMAND(*di));
	
	outb(C_START, COMMAND(*di));
	
	outb(di->irq, IRQSELECT(*di));

	outb(I_DOWNLOAD, IRQMASK(*di));
    
	outb(C_SET_DOWN, COMMAND(*di));

	di->state = INIT;
    
	restore_flags(flags);
    }
	
    return 0;
}
